"""
Lark Grammar
============

A reference implementation of the Lark grammar (using LALR(1))
"""
import lark
from pathlib import Path

examples_path = Path(__file__).parent
lark_path = Path(lark.__file__).parent

parser = lark.Lark.open(lark_path / 'grammars/lark.lark', rel_to=__file__, parser="lalr")


grammar_files = [
    examples_path / 'advanced/python2.lark',
    examples_path / 'relative-imports/multiples.lark',
    examples_path / 'relative-imports/multiple2.lark',
    examples_path / 'relative-imports/multiple3.lark',
    examples_path / 'tests/no_newline_at_end.lark',
    examples_path / 'tests/negative_priority.lark',
    examples_path / 'standalone/json.lark',
    lark_path / 'grammars/common.lark',
    lark_path / 'grammars/lark.lark',
    lark_path / 'grammars/unicode.lark',
    lark_path / 'grammars/python.lark',
]

def test():
    for grammar_file in grammar_files:
        tree = parser.parse(open(grammar_file).read())
    print("All grammars parsed successfully")

if __name__ == '__main__':
    test()



# coding=utf-8

import json
import sys
import unittest
from itertools import product
from unittest import TestCase

from lark import Lark
from lark.reconstruct import Reconstructor

common = """
%import common (WS_INLINE, NUMBER, WORD)
%ignore WS_INLINE
"""


def _remove_ws(s):
    return s.replace(' ', '').replace('\n', '')


class TestReconstructor(TestCase):

    def assert_reconstruct(self, grammar, code, **options):
        parser = Lark(grammar, parser='lalr', maybe_placeholders=False, **options)
        tree = parser.parse(code)
        new = Reconstructor(parser).reconstruct(tree)
        self.assertEqual(_remove_ws(code), _remove_ws(new))

    def test_starred_rule(self):
        g = """
        start: item*
        item: NL
            | rule
        rule: WORD ":" NUMBER
        NL: /(\\r?\\n)+\\s*/
        """ + common

        code = """
        Elephants: 12
        """

        self.assert_reconstruct(g, code)

    def test_starred_group(self):
        g = """
        start: (rule | NL)*
        rule: WORD ":" NUMBER
        NL: /(\\r?\\n)+\\s*/
        """ + common

        code = """
        Elephants: 12
        """

        self.assert_reconstruct(g, code)

    def test_alias(self):
        g = """
        start: line*
        line: NL
            | rule
            | "hello" -> hi
        rule: WORD ":" NUMBER
        NL: /(\\r?\\n)+\\s*/
        """ + common

        code = """
        Elephants: 12
        hello
        """

        self.assert_reconstruct(g, code)

    def test_keep_tokens(self):
        g = """
        start: (NL | stmt)*
        stmt: var op var
        !op: ("+" | "-" | "*" | "/")
        var: WORD
        NL: /(\\r?\\n)+\\s*/
        """ + common

        code = """
        a+b
        """

        self.assert_reconstruct(g, code)

    def test_expand_rule(self):
        g = """
        ?start: (NL | mult_stmt)*
        ?mult_stmt: sum_stmt ["*" sum_stmt]
        ?sum_stmt: var ["+" var]
        var: WORD
        NL: /(\\r?\\n)+\\s*/
        """ + common

        code = ['a', 'a*b', 'a+b', 'a*b+c', 'a+b*c', 'a+b*c+d']

        for c in code:
            self.assert_reconstruct(g, c)

    def test_json_example(self):
        test_json = '''
            {
                "empty_object" : {},
                "empty_array"  : [],
                "booleans"     : { "YES" : true, "NO" : false },
                "numbers"      : [ 0, 1, -2, 3.3, 4.4e5, 6.6e-7 ],
                "strings"      : [ "This", [ "And" , "That", "And a \\"b" ] ],
                "nothing"      : null
            }
        '''

        json_grammar = r"""
            ?start: value

            ?value: object
                  | array
                  | string
                  | SIGNED_NUMBER      -> number
                  | "true"             -> true
                  | "false"            -> false
                  | "null"             -> null

            array  : "[" [value ("," value)*] "]"
            object : "{" [pair ("," pair)*] "}"
            pair   : string ":" value

            string : ESCAPED_STRING

            %import common.ESCAPED_STRING
            %import common.SIGNED_NUMBER
            %import common.WS

            %ignore WS
        """

        json_parser = Lark(json_grammar, parser='lalr', maybe_placeholders=False)
        tree = json_parser.parse(test_json)

        new_json = Reconstructor(json_parser).reconstruct(tree)
        self.assertEqual(json.loads(new_json), json.loads(test_json))

    def test_keep_all_tokens(self):
        g = """
        start: "a"? _B? c? _d?
        _B: "b"
        c: "c"
        _d: "d"
        """
        examples = list(map(''.join, product(('', 'a'), ('', 'b'), ('', 'c'), ('', 'd'), )))
        for code in examples:
            self.assert_reconstruct(g, code, keep_all_tokens=True)

    @unittest.skipIf(sys.version_info < (3, 0), "Python 2 does not play well with Unicode.")
    def test_switch_grammar_unicode_terminal(self):
        """
        This test checks that a parse tree built with a grammar containing only ascii characters can be reconstructed
        with a grammar that has unicode rules (or vice versa). The original bug assigned ANON terminals to unicode
        keywords, which offsets the ANON terminal count in the unicode grammar and causes subsequent identical ANON
        tokens (e.g., `+=`) to mismatch between the two grammars.
        """

        g1 = """
        start: (NL | stmt)*
        stmt: "keyword" var op var
        !op: ("+=" | "-=" | "*=" | "/=")
        var: WORD
        NL: /(\\r?\\n)+\\s*/
        """ + common

        g2 = """
        start: (NL | stmt)*
        stmt: "குறிப்பு" var op var
        !op: ("+=" | "-=" | "*=" | "/=")
        var: WORD
        NL: /(\\r?\\n)+\\s*/
        """ + common

        code = """
        keyword x += y
        """

        l1 = Lark(g1, parser='lalr', maybe_placeholders=False)
        l2 = Lark(g2, parser='lalr', maybe_placeholders=False)
        r = Reconstructor(l2)

        tree = l1.parse(code)
        code2 = r.reconstruct(tree)
        assert l2.parse(code2) == tree


if __name__ == '__main__':
    unittest.main()



from unittest import TestCase, main

from lark import Token


class TestPatternMatching(TestCase):
    token = Token('A', 'a')

    def setUp(self):
        pass

    def test_matches_with_string(self):
        match self.token:
            case 'a':
                pass
            case _:
                assert False

    def test_matches_with_str_positional_arg(self):
        match self.token:
            case str('a'):
                pass
            case _:
                assert False

    def test_matches_with_token_positional_arg(self):
        match self.token:
            case Token('a'):
                assert False
            case Token('A'):
                pass
            case _:
                assert False

    def test_matches_with_token_kwarg_type(self):
        match self.token:
            case Token(type='A'):
                pass
            case _:
                assert False

    def test_matches_with_bad_token_type(self):
        match self.token:
            case Token(type='B'):
                assert False
            case _:
                pass



if __name__ == '__main__':
    main()



import logging
from contextlib import contextmanager
from lark import Lark, logger
from unittest import TestCase, main, skipIf

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

try:
    import interegular
except ImportError:
    interegular = None

@contextmanager
def capture_log():
    stream = StringIO()
    orig_handler = logger.handlers[0]
    del logger.handlers[:]
    logger.addHandler(logging.StreamHandler(stream))
    yield stream
    del logger.handlers[:]
    logger.addHandler(orig_handler)

class Testlogger(TestCase):

    def test_debug(self):
        logger.setLevel(logging.DEBUG)
        collision_grammar = '''
        start: as as
        as: a*
        a: "a"
        '''
        with capture_log() as log:
            Lark(collision_grammar, parser='lalr', debug=True)

        log = log.getvalue()
        # since there are conflicts about A
        # symbol A should appear in the log message for hint
        self.assertIn("A", log)

    def test_non_debug(self):
        logger.setLevel(logging.WARNING)
        collision_grammar = '''
        start: as as
        as: a*
        a: "a"
        '''
        with capture_log() as log:
            Lark(collision_grammar, parser='lalr', debug=False)
        log = log.getvalue()
        # no log message
        self.assertEqual(log, "")

    def test_loglevel_higher(self):
        logger.setLevel(logging.ERROR)
        collision_grammar = '''
        start: as as
        as: a*
        a: "a"
        '''
        with capture_log() as log:
            Lark(collision_grammar, parser='lalr', debug=True)
        log = log.getvalue()
        # no log message
        self.assertEqual(len(log), 0)

    @skipIf(interegular is None, "interegular is not installed, can't test regex collisions")
    def test_regex_collision(self):
        logger.setLevel(logging.WARNING)
        collision_grammar = '''
        start: A | B
        A: /a+/
        B: /(a|b)+/
        '''
        with capture_log() as log:
            Lark(collision_grammar, parser='lalr')

        log = log.getvalue()
        # since there are conflicts between A and B
        # symbols A and B should appear in the log message
        self.assertIn("A", log)
        self.assertIn("B", log)

    @skipIf(interegular is None, "interegular is not installed, can't test regex collisions")
    def test_no_regex_collision(self):
        logger.setLevel(logging.WARNING)
        collision_grammar = '''
        start: A " " B
        A: /a+/
        B: /(a|b)+/
        '''
        with capture_log() as log:
            Lark(collision_grammar, parser='lalr')

        log = log.getvalue()
        self.assertEqual(log, "")


if __name__ == '__main__':
    main()



from __future__ import absolute_import

import unittest
from copy import deepcopy

from lark import Lark, Tree, Token
from lark.exceptions import MissingVariableError
from lark.tree_templates import TemplateConf, Template, TemplateTranslator

SOME_NON_TEMPLATED_STRING = "foo bar"
SOME_TEMPLATE_NAME = "thing"
SOME_TEMPLATE_STRING = f"${SOME_TEMPLATE_NAME}"
SOME_NON_STRING = 12345
SOME_TEMPLATING_GRAMMAR = r"""
start: DASHES? foo DASHES? bar
DASHES: "--"
foo: "foo"
    | TEMPLATE_NAME -> var
bar: "bar"
    | TEMPLATE_NAME -> var
TEMPLATE_NAME: "$" NAME
NAME: /[^\W\d]\w*/
%ignore /[\t \f]+/  // WS
"""
SOME_FOO_TEMPLATE = f"{SOME_TEMPLATE_STRING} bar"
SOME_BAR_TEMPLATE = f"foo {SOME_TEMPLATE_STRING}"
SOME_NON_TEMPLATE_TREE = Tree("foo", children=["hi"])

__all__ = [
    "TestTreeTemplatesConf",
    "TestTreeTemplatesTemplateTranslator",
    "TestTreeTemplatesTemplate",
    "TestTreeTemplatesTemplateDefaultConf",
]


class TestTreeTemplatesConf(unittest.TestCase):
    parser = Lark(SOME_TEMPLATING_GRAMMAR)

    def test_conf_test_var__not_var(self):
        conf = TemplateConf(self.parser.parse)

        non_templates = {
            "non-templated string": SOME_NON_TEMPLATED_STRING,
            "non-var tree": Tree("stmt", children=[]),
            "var tree, non-templated string": Tree(
                "var", children=[SOME_NON_TEMPLATED_STRING]
            ),
            "var tree, templated string not first child": Tree(
                "var", children=[SOME_NON_TEMPLATED_STRING, SOME_TEMPLATE_STRING]
            ),
            "var tree, first child not string": Tree("var", children=[SOME_NON_STRING]),
            "var tree, no children": Tree("var", children=[]),
        }
        for description, test_case in non_templates.items():
            with self.subTest(msg=description):
                self.assertIsNone(conf.test_var(test_case))

    def test_conf_test_var__is_var(self):
        conf = TemplateConf(self.parser.parse)

        non_templates = {
            "templated string": SOME_TEMPLATE_STRING,
            "var tree, non-templated string": Tree(
                "var", children=[SOME_TEMPLATE_STRING]
            ),
        }
        for description, test_case in non_templates.items():
            with self.subTest(msg=description):
                self.assertEqual(SOME_TEMPLATE_NAME, conf.test_var(test_case))

    def test_conf_call__same_tree(self):
        conf = TemplateConf(self.parser.parse)
        explicitly_parsed = self.parser.parse(SOME_FOO_TEMPLATE)

        non_templates = {
            "to be parsed": SOME_FOO_TEMPLATE,
            "already parsed": explicitly_parsed,
        }
        for description, test_case in non_templates.items():
            with self.subTest(msg=description):
                template = conf(test_case)
                self.assertEqual(explicitly_parsed, template.tree)

    def test_template_match__default_conf_match_same_tree__empty_dictionary(self):
        template = Template(SOME_NON_TEMPLATE_TREE)

        self.assertEqual({}, template.match(SOME_NON_TEMPLATE_TREE))

    def test_template_match__only_tree(self):
        "This test might become irrelevant in the future"
        template_tree = Tree('bar', [Tree("var", children=["$foo"])])
        template = Template(template_tree)
        self.assertRaises(TypeError, template.match, Tree('bar', ['BAD']))


class TestTreeTemplatesTemplate(unittest.TestCase):
    parser = Lark(SOME_TEMPLATING_GRAMMAR)
    conf = TemplateConf(parser.parse)

    def test_template_match__same_tree_no_template__empty_dictionary(self):
        template = Template(SOME_NON_TEMPLATE_TREE, conf=self.conf)

        self.assertEqual({}, template.match(SOME_NON_TEMPLATE_TREE))

    def test_template_match__different_tree_no_template__none(self):
        template = Template(SOME_NON_TEMPLATE_TREE, conf=self.conf)

        self.assertIsNone(template.match(Tree("foo", children=["bye"])))

    def test_template_match__no_template__empty_dictionary(self):
        tree = self.parser.parse(SOME_NON_TEMPLATED_STRING)
        template = Template(tree, conf=self.conf)

        non_templates = {
            "un-parsed string": SOME_NON_TEMPLATED_STRING,
            "parsed tree": tree,
        }
        for description, test_case in non_templates.items():
            with self.subTest(msg=description):
                self.assertEqual({}, template.match(test_case))

    def test_template_match__with_template__empty_dictionary(self):
        tree = self.parser.parse(SOME_FOO_TEMPLATE)
        template = Template(tree, conf=self.conf)

        non_templates = {"un-parsed string": SOME_FOO_TEMPLATE, "parsed tree": tree}
        expected_result = {SOME_TEMPLATE_NAME: tree.children[0]}

        for description, test_case in non_templates.items():
            with self.subTest(msg=description):
                self.assertEqual(expected_result, template.match(test_case))

    def test_template_match__different_tree__none(self):
        tree = self.parser.parse(SOME_FOO_TEMPLATE)
        template = Template(tree, conf=self.conf)

        non_templates = {
            "un-parsed string": SOME_BAR_TEMPLATE,
            "parsed tree": self.parser.parse(SOME_BAR_TEMPLATE),
        }
        for description, test_case in non_templates.items():
            with self.subTest(msg=description):
                self.assertIsNone(template.match(test_case))

    def test_template_search__same_tree_no_template__empty_generator(self):
        template = Template(SOME_NON_TEMPLATE_TREE, conf=self.conf)

        self.assertEqual([], list(template.search(SOME_NON_TEMPLATE_TREE)))

    def test_template_search__same_tree_as_child__empty_generator(self):
        template = Template(SOME_NON_TEMPLATE_TREE, conf=self.conf)

        self.assertEqual(
            [], list(template.search(Tree("root", children=[SOME_NON_TEMPLATE_TREE])))
        )

    def test_template_search__with_template__matched_result_with_parent_tree(self):
        tree = self.parser.parse(SOME_FOO_TEMPLATE)
        template = Template(tree, conf=self.conf)

        non_templates = {"un-parsed string": SOME_FOO_TEMPLATE, "parsed tree": tree}
        expected_result = [(tree, {SOME_TEMPLATE_NAME: tree.children[0]})]

        for description, test_case in non_templates.items():
            with self.subTest(msg=description):
                self.assertEqual(expected_result, list(template.search(test_case)))

    def test_template_apply_vars__empty__exception(self):
        tree = self.parser.parse(SOME_FOO_TEMPLATE)
        template = Template(tree, conf=self.conf)

        with self.assertRaises(MissingVariableError):
            template.apply_vars({})

    def test_template_apply_vars__no_matching_vars__exception(self):
        tree = self.parser.parse(SOME_FOO_TEMPLATE)
        template = Template(tree, conf=self.conf)

        with self.assertRaises(MissingVariableError):
            template.apply_vars({"not used": SOME_NON_TEMPLATE_TREE})

    def test_template_apply_vars__matching_vars__template_replaced(self):
        tree = self.parser.parse(SOME_FOO_TEMPLATE)
        template = Template(tree, conf=self.conf)

        expected_result = deepcopy(tree)
        expected_result.children[0] = SOME_NON_TEMPLATE_TREE
        self.assertEqual(
            expected_result,
            template.apply_vars({SOME_TEMPLATE_NAME: SOME_NON_TEMPLATE_TREE}),
        )


class TestTreeTemplatesTemplateTranslator(unittest.TestCase):
    parser = Lark(SOME_TEMPLATING_GRAMMAR)
    conf = TemplateConf(parser.parse)

    def test_translate__empty_translations__same_tree(self):
        # no translations to match, so doesn't replace anything & can't error
        translator = TemplateTranslator({})
        tree = self.parser.parse(SOME_FOO_TEMPLATE)

        expected_result = deepcopy(tree)
        self.assertEqual(expected_result, translator.translate(tree))

    def test_translate__one_translations__same_tree(self):
        translations = {
            self.conf(f"${SOME_TEMPLATE_NAME} bar"): self.conf(
                f"--${SOME_TEMPLATE_NAME}-- bar"
            )
        }
        translator = TemplateTranslator(translations)
        tree = self.parser.parse(SOME_NON_TEMPLATED_STRING)

        expected_result = deepcopy(tree)
        expected_result.children.insert(0, Token("DASHES", "--"))
        expected_result.children.insert(2, Token("DASHES", "--"))
        self.assertEqual(expected_result, translator.translate(tree))


class TestTreeTemplatesTemplateDefaultConf(unittest.TestCase):
    def test_template_match__match_same_tree__empty_dictionary(self):
        tree = Tree("foo", children=["hi"])
        template = Template(tree)

        self.assertEqual({}, template.match(tree))


if __name__ == "__main__":
    unittest.main()



from __future__ import absolute_import

import os
from unittest import TestCase, main

from lark import Lark, Token, Tree, ParseError, UnexpectedInput
from lark.load_grammar import GrammarError, GRAMMAR_ERRORS, find_grammar_errors, list_grammar_imports
from lark.load_grammar import FromPackageLoader


class TestGrammar(TestCase):
    def setUp(self):
        pass

    def test_errors(self):
        for msg, examples in GRAMMAR_ERRORS:
            for example in examples:
                try:
                    p = Lark(example)
                except GrammarError as e:
                    assert msg in str(e)
                else:
                    assert False, "example did not raise an error"

    def test_empty_literal(self):
        # Issues #888
        self.assertRaises(GrammarError, Lark, "start: \"\"")

    def test_ignore_name(self):
        spaces = []
        p = Lark("""
            start: "a" "b"
            WS: " "
            %ignore WS
        """, parser='lalr', lexer_callbacks={'WS': spaces.append})
        assert p.parse("a b") == p.parse("a    b")
        assert len(spaces) == 5


    def test_override_rule(self):
        # Overrides the 'sep' template in existing grammar to add an optional terminating delimiter
        # Thus extending it beyond its original capacity
        p = Lark("""
            %import .test_templates_import (start, sep)

            %override sep{item, delim}: item (delim item)* delim?
            %ignore " "
        """, source_path=__file__)

        a = p.parse('[1, 2, 3]')
        b = p.parse('[1, 2, 3, ]')
        assert a == b

        self.assertRaises(GrammarError, Lark, """
            %import .test_templates_import (start, sep)

            %override sep{item}: item (delim item)* delim?
        """, source_path=__file__)

        self.assertRaises(GrammarError, Lark, """
            %override sep{item}: item (delim item)* delim?
        """, source_path=__file__)

    def test_override_terminal(self):
        p = Lark("""

            %import .grammars.ab (startab, A, B)

            %override A: "c"
            %override B: "d"
        """, start='startab', source_path=__file__)

        a = p.parse('cd')
        self.assertEqual(a.children[0].children, [Token('A', 'c'), Token('B', 'd')])

    def test_extend_rule(self):
        p = Lark("""
            %import .grammars.ab (startab, A, B, expr)

            %extend expr: B A
        """, start='startab', source_path=__file__)
        a = p.parse('abab')
        self.assertEqual(a.children[0].children, ['a', Tree('expr', ['b', 'a']), 'b'])

        self.assertRaises(GrammarError, Lark, """
            %extend expr: B A
        """)

    def test_extend_term(self):
        p = Lark("""
            %import .grammars.ab (startab, A, B, expr)

            %extend A: "c"
        """, start='startab', source_path=__file__)
        a = p.parse('acbb')
        self.assertEqual(a.children[0].children, ['a', Tree('expr', ['c', 'b']), 'b'])

    def test_extend_twice(self):
        p = Lark("""
            start: x+

            x: "a"
            %extend x: "b"
            %extend x: "c"
        """)

        assert p.parse("abccbba") == p.parse("cbabbbb")

    def test_undefined_ignore(self):
        g = """!start: "A"

            %ignore B
            """
        self.assertRaises( GrammarError, Lark, g)

        g = """!start: "A"

            %ignore start
            """
        self.assertRaises( GrammarError, Lark, g)

    def test_alias_in_terminal(self):
        g = """start: TERM
            TERM: "a" -> alias
            """
        self.assertRaises( GrammarError, Lark, g)

    def test_undefined_rule(self):
        self.assertRaises(GrammarError, Lark, """start: a""")

    def test_undefined_term(self):
        self.assertRaises(GrammarError, Lark, """start: A""")

    def test_token_multiline_only_works_with_x_flag(self):
        g = r"""start: ABC
                ABC: /  a      b c
                            d
                            e f
                        /i
                    """
        self.assertRaises( GrammarError, Lark, g)

    def test_import_custom_sources(self):
        custom_loader = FromPackageLoader(__name__, ('grammars', ))

        grammar = """
        start: startab

        %import ab.startab
        """

        p = Lark(grammar, import_paths=[custom_loader])
        self.assertEqual(p.parse('ab'),
                            Tree('start', [Tree('startab', [Tree('ab__expr', [Token('ab__A', 'a'), Token('ab__B', 'b')])])]))

    def test_import_custom_sources2(self):
        custom_loader = FromPackageLoader(__name__, ('grammars', ))

        grammar = """
        start: rule_to_import

        %import test_relative_import_of_nested_grammar__grammar_to_import.rule_to_import
        """
        p = Lark(grammar, import_paths=[custom_loader])
        x = p.parse('N')
        self.assertEqual(next(x.find_data('rule_to_import')).children, ['N'])

    def test_import_custom_sources3(self):
        custom_loader2 = FromPackageLoader(__name__)
        grammar = """
        %import .test_relative_import (start, WS)
        %ignore WS
        """
        p = Lark(grammar, import_paths=[custom_loader2], source_path=__file__) # import relative to current file
        x = p.parse('12 capybaras')
        self.assertEqual(x.children, ['12', 'capybaras'])

    def test_find_grammar_errors(self):
        text = """
        a: rule
        b rule
        c: rule
        B.: "hello" f
        D: "okay"
        """

        assert [e.line for e, _s in find_grammar_errors(text)] == [3, 5]

        text = """
        a: rule
        b rule
        | ok
        c: rule
        B.: "hello" f
        D: "okay"
        """

        assert [e.line for e, _s in find_grammar_errors(text)] == [3, 4, 6]

        text = """
        a: rule @#$#@$@&&
        b: rule
        | ok
        c: rule
        B: "hello" f @
        D: "okay"
        """

        x = find_grammar_errors(text)
        assert [e.line for e, _s in find_grammar_errors(text)] == [2, 6]

    def test_ranged_repeat_terms(self):
        g = u"""!start: AAA
                AAA: "A"~3
            """
        l = Lark(g, parser='lalr')
        self.assertEqual(l.parse(u'AAA'), Tree('start', ["AAA"]))
        self.assertRaises((ParseError, UnexpectedInput), l.parse, u'AA')
        self.assertRaises((ParseError, UnexpectedInput), l.parse, u'AAAA')

        g = u"""!start: AABB CC
                AABB: "A"~0..2 "B"~2
                CC: "C"~1..2
            """
        l = Lark(g, parser='lalr')
        self.assertEqual(l.parse(u'AABBCC'), Tree('start', ['AABB', 'CC']))
        self.assertEqual(l.parse(u'BBC'), Tree('start', ['BB', 'C']))
        self.assertEqual(l.parse(u'ABBCC'), Tree('start', ['ABB', 'CC']))
        self.assertRaises((ParseError, UnexpectedInput), l.parse, u'AAAB')
        self.assertRaises((ParseError, UnexpectedInput), l.parse, u'AAABBB')
        self.assertRaises((ParseError, UnexpectedInput), l.parse, u'ABB')
        self.assertRaises((ParseError, UnexpectedInput), l.parse, u'AAAABB')

    def test_ranged_repeat_large(self):
        g = u"""!start: "A"~60
            """
        l = Lark(g, parser='lalr')
        self.assertGreater(len(l.rules), 1, "Expected that more than one rule will be generated")
        self.assertEqual(l.parse(u'A' * 60), Tree('start', ["A"] * 60))
        self.assertRaises(ParseError, l.parse, u'A' * 59)
        self.assertRaises((ParseError, UnexpectedInput), l.parse, u'A' * 61)

        g = u"""!start: "A"~15..100
            """
        l = Lark(g, parser='lalr')
        for i in range(0, 110):
            if 15 <= i <= 100:
                self.assertEqual(l.parse(u'A' * i), Tree('start', ['A']*i))
            else:
                self.assertRaises(UnexpectedInput, l.parse, u'A' * i)

        # 8191 is a Mersenne prime
        g = u"""start: "A"~8191
            """
        l = Lark(g, parser='lalr')
        self.assertEqual(l.parse(u'A' * 8191), Tree('start', []))
        self.assertRaises(UnexpectedInput, l.parse, u'A' * 8190)
        self.assertRaises(UnexpectedInput, l.parse, u'A' * 8192)

    def test_large_terminal(self):
        g = "start: NUMBERS\n"
        g += "NUMBERS: " + '|'.join('"%s"' % i for i in range(0, 1000))

        l = Lark(g, parser='lalr')
        for i in (0, 9, 99, 999):
            self.assertEqual(l.parse(str(i)), Tree('start', [str(i)]))
        for i in (-1, 1000):
            self.assertRaises(UnexpectedInput, l.parse, str(i))

    def test_list_grammar_imports(self):
            grammar = """
            %import .test_templates_import (start, sep)

            %override sep{item, delim}: item (delim item)* delim?
            %ignore " "
            """

            imports = list_grammar_imports(grammar, [os.path.dirname(__file__)])
            self.assertEqual({os.path.split(i)[-1] for i in imports}, {'test_templates_import.lark', 'templates.lark'})

            imports = list_grammar_imports('%import common.WS', [])
            assert len(imports) == 1 and imports[0].pkg_name == 'lark'

    def test_inline_with_expand_single(self):
        grammar = r"""
        start: _a
        !?_a: "A"
        """
        self.assertRaises(GrammarError, Lark, grammar)


    def test_line_breaks(self):
        p = Lark(r"""start: "a" \
                       "b"
                """)
        p.parse('ab')






if __name__ == '__main__':
    main()



from __future__ import absolute_import

import unittest

from lark import Lark
from lark.lexer import Token
from lark.tree import Tree
from lark.visitors import Visitor, Transformer, Discard
from lark.parsers.earley_forest import TreeForestTransformer, handles_ambiguity

class TestTreeForestTransformer(unittest.TestCase):

    grammar = """
    start: ab bc cd
    !ab: "A" "B"?
    !bc: "B"? "C"?
    !cd: "C"? "D"
    """

    parser = Lark(grammar, parser='earley', ambiguity='forest')
    forest = parser.parse("ABCD")

    def test_identity_resolve_ambiguity(self):
        l = Lark(self.grammar, parser='earley', ambiguity='resolve')
        tree1 = l.parse("ABCD")
        tree2 = TreeForestTransformer(resolve_ambiguity=True).transform(self.forest)
        self.assertEqual(tree1, tree2)

    def test_identity_explicit_ambiguity(self):
        l = Lark(self.grammar, parser='earley', ambiguity='explicit')
        tree1 = l.parse("ABCD")
        tree2 = TreeForestTransformer(resolve_ambiguity=False).transform(self.forest)
        self.assertEqual(tree1, tree2)

    def test_tree_class(self):

        class CustomTree(Tree):
            pass

        class TreeChecker(Visitor):
            def __default__(self, tree):
                assert isinstance(tree, CustomTree)

        tree = TreeForestTransformer(resolve_ambiguity=False, tree_class=CustomTree).transform(self.forest)
        TreeChecker().visit(tree)

    def test_token_calls(self):

        visited = [False] * 4

        class CustomTransformer(TreeForestTransformer):
            def A(self, node):
                assert node.type == 'A'
                visited[0] = True
            def B(self, node):
                assert node.type == 'B'
                visited[1] = True
            def C(self, node):
                assert node.type == 'C'
                visited[2] = True
            def D(self, node):
                assert node.type == 'D'
                visited[3] = True

        tree = CustomTransformer(resolve_ambiguity=False).transform(self.forest)
        assert visited == [True] * 4

    def test_default_token(self):

        token_count = [0]

        class CustomTransformer(TreeForestTransformer):
            def __default_token__(self, node):
                token_count[0] += 1
                assert isinstance(node, Token)

        tree = CustomTransformer(resolve_ambiguity=True).transform(self.forest)
        self.assertEqual(token_count[0], 4)

    def test_rule_calls(self):

        visited_start = [False]
        visited_ab = [False]
        visited_bc = [False]
        visited_cd = [False]

        class CustomTransformer(TreeForestTransformer):
            def start(self, data):
                visited_start[0] = True
            def ab(self, data):
                visited_ab[0] = True
            def bc(self, data):
                visited_bc[0] = True
            def cd(self, data):
                visited_cd[0] = True

        tree = CustomTransformer(resolve_ambiguity=False).transform(self.forest)
        self.assertTrue(visited_start[0])
        self.assertTrue(visited_ab[0])
        self.assertTrue(visited_bc[0])
        self.assertTrue(visited_cd[0])

    def test_default_rule(self):

        rule_count = [0]

        class CustomTransformer(TreeForestTransformer):
            def __default__(self, name, data):
                rule_count[0] += 1

        tree = CustomTransformer(resolve_ambiguity=True).transform(self.forest)
        self.assertEqual(rule_count[0], 4)

    def test_default_ambig(self):

        ambig_count = [0]

        class CustomTransformer(TreeForestTransformer):
            def __default_ambig__(self, name, data):
                if len(data) > 1:
                    ambig_count[0] += 1

        tree = CustomTransformer(resolve_ambiguity=False).transform(self.forest)
        self.assertEqual(ambig_count[0], 1)

    def test_handles_ambiguity(self):

        class CustomTransformer(TreeForestTransformer):
            @handles_ambiguity
            def start(self, data):
                assert isinstance(data, list)
                assert len(data) == 4
                for tree in data:
                    assert tree.data == 'start'
                return 'handled'

            @handles_ambiguity
            def ab(self, data):
                assert isinstance(data, list)
                assert len(data) == 1
                assert data[0].data == 'ab'

        tree = CustomTransformer(resolve_ambiguity=False).transform(self.forest)
        self.assertEqual(tree, 'handled')

    def test_discard(self):

        class CustomTransformer(TreeForestTransformer):
            def bc(self, data):
                return Discard

            def D(self, node):
                return Discard

        class TreeChecker(Transformer):
            def bc(self, children):
                assert False

            def D(self, token):
                assert False

        tree = CustomTransformer(resolve_ambiguity=False).transform(self.forest)
        TreeChecker(visit_tokens=True).transform(tree)

    def test_aliases(self):

        visited_ambiguous = [False]
        visited_full = [False]

        class CustomTransformer(TreeForestTransformer):
            @handles_ambiguity
            def start(self, data):
                for tree in data:
                    assert tree.data == 'ambiguous' or tree.data == 'full'

            def ambiguous(self, data):
                visited_ambiguous[0] = True
                assert len(data) == 3
                assert data[0].data == 'ab'
                assert data[1].data == 'bc'
                assert data[2].data == 'cd'
                return self.tree_class('ambiguous', data)

            def full(self, data):
                visited_full[0] = True
                assert len(data) == 1
                assert data[0].data == 'abcd'
                return self.tree_class('full', data)

        grammar = """
        start: ab bc cd -> ambiguous
            | abcd -> full
        !ab: "A" "B"?
        !bc: "B"? "C"?
        !cd: "C"? "D"
        !abcd: "ABCD"
        """

        l = Lark(grammar, parser='earley', ambiguity='forest')
        forest = l.parse('ABCD')
        tree = CustomTransformer(resolve_ambiguity=False).transform(forest)
        self.assertTrue(visited_ambiguous[0])
        self.assertTrue(visited_full[0])

    def test_transformation(self):

        class CustomTransformer(TreeForestTransformer):
            def __default__(self, name, data):
                result = []
                for item in data:
                    if isinstance(item, list):
                        result += item
                    else:
                        result.append(item)
                return result

            def __default_token__(self, node):
                return node.lower()

            def __default_ambig__(self, name, data):
                return data[0]

        result = CustomTransformer(resolve_ambiguity=False).transform(self.forest)
        expected = ['a', 'b', 'c', 'd']
        self.assertEqual(result, expected)

if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, print_function

import unittest
import logging
import sys
from lark import logger

from .test_trees import TestTrees
from .test_tools import TestStandalone
from .test_cache import TestCache
from .test_grammar import TestGrammar
from .test_reconstructor import TestReconstructor
from .test_tree_forest_transformer import TestTreeForestTransformer
from .test_lexer import TestLexer
from .test_python_grammar import TestPythonParser
from .test_tree_templates import *  # We define __all__ to list which TestSuites to run

try:
    from .test_nearley.test_nearley import TestNearley
except ImportError:
    logger.warning("Warning: Skipping tests for Nearley grammar imports (js2py required)")

# from .test_selectors import TestSelectors
# from .test_grammars import TestPythonG, TestConfigG

from .test_logger import Testlogger

from .test_parser import *  # We define __all__ to list which TestSuites to run

if sys.version_info >= (3, 10):
    from .test_pattern_matching import TestPatternMatching

logger.setLevel(logging.INFO)

if __name__ == '__main__':
    unittest.main()



from unittest import TestCase, main

from lark import Lark, Tree

class TestLexer(TestCase):
    def setUp(self):
        pass

    def test_basic(self):
        p = Lark("""
            start: "a" "b" "c" "d"
            %ignore " "
        """)

        res = list(p.lex("abc cba dd"))
        assert res == list('abccbadd')

        res = list(p.lex("abc cba dd", dont_ignore=True))
        assert res == list('abc cba dd')


if __name__ == '__main__':
    main()
